from typing import List
from typing import Optional
from typing import Union

from pyspark.sql.types import ArrayType
from pyspark.sql.types import DataType
from pyspark.sql.types import StringType
from pyspark.sql.types import StructField
from pyspark.sql.types import StructType


# This file is auto-generated by generate_schema so do not edit manually
# noinspection PyPep8Naming
class AllergyIntoleranceSchema:
    """
    Risk of harmful or undesirable, physiological response which is unique to an
    individual and associated with exposure to a substance.
    """

    # noinspection PyDefaultArgument
    @staticmethod
    def get_schema(
        max_nesting_depth: Optional[int] = 6,
        nesting_depth: int = 0,
        nesting_list: List[str] = [],
        max_recursion_limit: Optional[int] = 2,
        include_extension: Optional[bool] = False,
    ) -> Union[StructType, DataType]:
        """
        Risk of harmful or undesirable, physiological response which is unique to an
        individual and associated with exposure to a substance.


        resourceType: This is a AllergyIntolerance resource

        identifier: This records identifiers associated with this allergy/intolerance concern that
            are defined by business processes and/or used to refer to it when a direct URL
            reference to the resource itself is not appropriate (e.g. in CDA documents, or
            in written / printed documentation).

        clinicalStatus: The clinical status of the allergy or intolerance.

        verificationStatus: Assertion about certainty associated with the propensity, or potential risk,
            of a reaction to the identified substance (including pharmaceutical product).

        type: Identification of the underlying physiological mechanism for the reaction
            risk.

        category: Category of the identified substance.

        criticality: Estimate of the potential clinical harm, or seriousness, of the reaction to
            the identified substance.

        code: Code for an allergy or intolerance statement (either a positive or a
            negated/excluded statement).  This may be a code for a substance or
            pharmaceutical product that is considered to be responsible for the adverse
            reaction risk (e.g., "Latex"), an allergy or intolerance condition (e.g.,
            "Latex allergy"), or a negated/excluded code for a specific substance or class
            (e.g., "No latex allergy") or a general or categorical negated statement
            (e.g.,  "No known allergy", "No known drug allergies").

        patient: The patient who has the allergy or intolerance.

        onsetDateTime: Estimated or actual date,  date-time, or age when allergy or intolerance was
            identified.

        onsetAge: Estimated or actual date,  date-time, or age when allergy or intolerance was
            identified.

        onsetPeriod: Estimated or actual date,  date-time, or age when allergy or intolerance was
            identified.

        onsetRange: Estimated or actual date,  date-time, or age when allergy or intolerance was
            identified.

        onsetString: Estimated or actual date,  date-time, or age when allergy or intolerance was
            identified.

        assertedDate: The date on which the existance of the AllergyIntolerance was first asserted
            or acknowledged.

        recorder: Individual who recorded the record and takes responsibility for its content.

        asserter: The source of the information about the allergy that is recorded.

        lastOccurrence: Represents the date and/or time of the last known occurrence of a reaction
            event.

        note: Additional narrative about the propensity for the Adverse Reaction, not
            captured in other fields.

        reaction: Details about each adverse reaction event linked to exposure to the identified
            substance.

        """
        from spark_fhir_schemas.stu3.complex_types.identifier import IdentifierSchema
        from spark_fhir_schemas.stu3.complex_types.codeableconcept import (
            CodeableConceptSchema,
        )
        from spark_fhir_schemas.stu3.complex_types.reference import ReferenceSchema
        from spark_fhir_schemas.stu3.complex_types.age import AgeSchema
        from spark_fhir_schemas.stu3.complex_types.period import PeriodSchema
        from spark_fhir_schemas.stu3.complex_types.range import RangeSchema
        from spark_fhir_schemas.stu3.complex_types.annotation import AnnotationSchema
        from spark_fhir_schemas.stu3.complex_types.allergyintolerance_reaction import (
            AllergyIntolerance_ReactionSchema,
        )

        if (
            max_recursion_limit
            and nesting_list.count("AllergyIntolerance") >= max_recursion_limit
        ) or (max_nesting_depth and nesting_depth >= max_nesting_depth):
            return StructType([StructField("id", StringType(), True)])
        # add my name to recursion list for later
        my_nesting_list: List[str] = nesting_list + ["AllergyIntolerance"]
        schema = StructType(
            [
                # This is a AllergyIntolerance resource
                StructField("resourceType", StringType(), True),
                # This records identifiers associated with this allergy/intolerance concern that
                # are defined by business processes and/or used to refer to it when a direct URL
                # reference to the resource itself is not appropriate (e.g. in CDA documents, or
                # in written / printed documentation).
                StructField(
                    "identifier",
                    ArrayType(
                        IdentifierSchema.get_schema(
                            max_nesting_depth=max_nesting_depth,
                            nesting_depth=nesting_depth + 1,
                            nesting_list=my_nesting_list,
                            max_recursion_limit=max_recursion_limit,
                            include_extension=include_extension,
                        )
                    ),
                    True,
                ),
                # The clinical status of the allergy or intolerance.
                StructField("clinicalStatus", StringType(), True),
                # Assertion about certainty associated with the propensity, or potential risk,
                # of a reaction to the identified substance (including pharmaceutical product).
                StructField("verificationStatus", StringType(), True),
                # Identification of the underlying physiological mechanism for the reaction
                # risk.
                StructField("type", StringType(), True),
                # Category of the identified substance.
                # Estimate of the potential clinical harm, or seriousness, of the reaction to
                # the identified substance.
                StructField("criticality", StringType(), True),
                # Code for an allergy or intolerance statement (either a positive or a
                # negated/excluded statement).  This may be a code for a substance or
                # pharmaceutical product that is considered to be responsible for the adverse
                # reaction risk (e.g., "Latex"), an allergy or intolerance condition (e.g.,
                # "Latex allergy"), or a negated/excluded code for a specific substance or class
                # (e.g., "No latex allergy") or a general or categorical negated statement
                # (e.g.,  "No known allergy", "No known drug allergies").
                StructField(
                    "code",
                    CodeableConceptSchema.get_schema(
                        max_nesting_depth=max_nesting_depth,
                        nesting_depth=nesting_depth + 1,
                        nesting_list=my_nesting_list,
                        max_recursion_limit=max_recursion_limit,
                        include_extension=include_extension,
                    ),
                    True,
                ),
                # The patient who has the allergy or intolerance.
                StructField(
                    "patient",
                    ReferenceSchema.get_schema(
                        max_nesting_depth=max_nesting_depth,
                        nesting_depth=nesting_depth + 1,
                        nesting_list=my_nesting_list,
                        max_recursion_limit=max_recursion_limit,
                        include_extension=include_extension,
                    ),
                    True,
                ),
                # Estimated or actual date,  date-time, or age when allergy or intolerance was
                # identified.
                StructField("onsetDateTime", StringType(), True),
                # Estimated or actual date,  date-time, or age when allergy or intolerance was
                # identified.
                StructField(
                    "onsetAge",
                    AgeSchema.get_schema(
                        max_nesting_depth=max_nesting_depth,
                        nesting_depth=nesting_depth + 1,
                        nesting_list=my_nesting_list,
                        max_recursion_limit=max_recursion_limit,
                        include_extension=include_extension,
                    ),
                    True,
                ),
                # Estimated or actual date,  date-time, or age when allergy or intolerance was
                # identified.
                StructField(
                    "onsetPeriod",
                    PeriodSchema.get_schema(
                        max_nesting_depth=max_nesting_depth,
                        nesting_depth=nesting_depth + 1,
                        nesting_list=my_nesting_list,
                        max_recursion_limit=max_recursion_limit,
                        include_extension=include_extension,
                    ),
                    True,
                ),
                # Estimated or actual date,  date-time, or age when allergy or intolerance was
                # identified.
                StructField(
                    "onsetRange",
                    RangeSchema.get_schema(
                        max_nesting_depth=max_nesting_depth,
                        nesting_depth=nesting_depth + 1,
                        nesting_list=my_nesting_list,
                        max_recursion_limit=max_recursion_limit,
                        include_extension=include_extension,
                    ),
                    True,
                ),
                # Estimated or actual date,  date-time, or age when allergy or intolerance was
                # identified.
                StructField("onsetString", StringType(), True),
                # The date on which the existance of the AllergyIntolerance was first asserted
                # or acknowledged.
                StructField("assertedDate", StringType(), True),
                # Individual who recorded the record and takes responsibility for its content.
                StructField(
                    "recorder",
                    ReferenceSchema.get_schema(
                        max_nesting_depth=max_nesting_depth,
                        nesting_depth=nesting_depth + 1,
                        nesting_list=my_nesting_list,
                        max_recursion_limit=max_recursion_limit,
                        include_extension=include_extension,
                    ),
                    True,
                ),
                # The source of the information about the allergy that is recorded.
                StructField(
                    "asserter",
                    ReferenceSchema.get_schema(
                        max_nesting_depth=max_nesting_depth,
                        nesting_depth=nesting_depth + 1,
                        nesting_list=my_nesting_list,
                        max_recursion_limit=max_recursion_limit,
                        include_extension=include_extension,
                    ),
                    True,
                ),
                # Represents the date and/or time of the last known occurrence of a reaction
                # event.
                StructField("lastOccurrence", StringType(), True),
                # Additional narrative about the propensity for the Adverse Reaction, not
                # captured in other fields.
                StructField(
                    "note",
                    ArrayType(
                        AnnotationSchema.get_schema(
                            max_nesting_depth=max_nesting_depth,
                            nesting_depth=nesting_depth + 1,
                            nesting_list=my_nesting_list,
                            max_recursion_limit=max_recursion_limit,
                            include_extension=include_extension,
                        )
                    ),
                    True,
                ),
                # Details about each adverse reaction event linked to exposure to the identified
                # substance.
                StructField(
                    "reaction",
                    ArrayType(
                        AllergyIntolerance_ReactionSchema.get_schema(
                            max_nesting_depth=max_nesting_depth,
                            nesting_depth=nesting_depth + 1,
                            nesting_list=my_nesting_list,
                            max_recursion_limit=max_recursion_limit,
                            include_extension=include_extension,
                        )
                    ),
                    True,
                ),
            ]
        )
        if not include_extension:
            schema.fields = [
                c
                if c.name != "extension"
                else StructField("extension", StringType(), True)
                for c in schema.fields
            ]
        return schema
